package com.nisum.ecomcustomer.model;

import com.nisum.ecomcustomer.validators.CustomerAddressesConstraint;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.annotation.*;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.data.mongodb.core.mapping.Field;

import javax.validation.Valid;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

@NoArgsConstructor
@Data
@TypeAlias("customer")
@Document("customers")
public class Customer {

    @Transient
    public static final String SEQUENCE_NAME = "customers_sequence";

    @Id
    private Long id;
    @Field("fName")
    @NotBlank(message = "first name can't be blank")
    private String firstName;
    @Field("lName")
    private String lastName;
    @NotBlank(message = "email is mandatory")
    @Email
    @Indexed(unique = true, name = "email_index")
    private String email;
    @NotBlank(message = "dayPhone can't be blank")
    private String dayPhone;
    @Field("type")
    private CustomerType customerType;
    @CustomerAddressesConstraint
    @Valid
    private List<Address> addresses;
    @CreatedDate
    private LocalDate createdDate;
    @LastModifiedDate
    private LocalDateTime modifiedDate;
}
