package com.nisum.ecomcustomer.validators;

import com.nisum.ecomcustomer.model.Address;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.util.List;

public class CustomerAddressesValidator implements ConstraintValidator<CustomerAddressesConstraint, List<Address>> {

    private ConstraintValidatorContext validatorContext;

    @Override
    public boolean isValid(List<Address> addresses, ConstraintValidatorContext context) {
        validatorContext = context;
        return hasValidSize(addresses) && hasAtLeastAndOnlyOneBillingAddress(addresses) && hasAtLeastOneAndOnlyOneDefaultShippingAddress(addresses);
    }

    private boolean hasValidSize(List<Address> addresses) {
        boolean hasValidSize = addresses.size() <= 5 && addresses.size() > 1;
        if (!hasValidSize) {
            addValidationMessage("Must contain only one billing address and at least one shipping address. At most 5 addresses are allowed");
        }
        return hasValidSize;
    }

    private boolean hasAtLeastAndOnlyOneBillingAddress(List<Address> addresses) {
        boolean hasAtLeastAndOnlyOneBillingAddress = addresses.stream().filter(address -> address.getAddressType().getValue().equals("bill")).count() == 1;
        if (!hasAtLeastAndOnlyOneBillingAddress) {
            addValidationMessage("Must contain only one billing address and at least one shipping address.");
        }
        return hasAtLeastAndOnlyOneBillingAddress;
    }

    private boolean hasAtLeastOneAndOnlyOneDefaultShippingAddress(List<Address> addresses) {
        boolean hasAtLeastOneShippingAddress = addresses.stream().anyMatch(address -> address.getAddressType().getValue().equals("ship"));
        boolean hasOnlyOneDefaultShippingAddress = false;
        if (hasAtLeastOneShippingAddress) {
            hasOnlyOneDefaultShippingAddress = addresses.stream().filter(address -> address.getAddressType().getValue().equals("ship") && address.isDefault()).count() == 1;
            if (!hasOnlyOneDefaultShippingAddress) {
                addValidationMessage("there must be only one default shipping address");
            }
        } else {
            addValidationMessage("Must contain only one billing address and at least one shipping address.");
        }
        return hasAtLeastOneShippingAddress && hasOnlyOneDefaultShippingAddress;
    }

    private void addValidationMessage(String message) {
        validatorContext.disableDefaultConstraintViolation(); // disable default violation message
        validatorContext.buildConstraintViolationWithTemplate(message).addConstraintViolation();
    }
}
